<?php

namespace Drupal\feeds_tamper_media_url\Plugin\Tamper;

use Drupal\Core\Form\FormStateInterface;
use Drupal\tamper\Exception\TamperException;
use Drupal\tamper\TamperableItemInterface;
use Drupal\tamper\TamperBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Entity\EntityStorageException;
use GuzzleHttp\Client;
use Drupal\Core\File\Exception\FileException;
use Drupal\media\Entity\Media;
use Drupal\media\Entity\MediaType;

/**
 * Plugin implementation of the find_replace plugin.
 *
 * @Tamper(
 *   id = "create_media_tamper",
 *   label = @Translation("Create Media Tamper"),
 *   description = @Translation("Create Media Tamper"),
 *   category = "Other"
 * )
 */
class CreateMedia extends TamperBase
{

  const SETTING_MEDIA_TYPE = 'media_type';
  const SETTING_MEDIA_FIELD = 'media_field';

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration()
  {
    $config = parent::defaultConfiguration();
    $config[self::SETTING_MEDIA_TYPE] = '';
    $config[self::SETTING_MEDIA_FIELD] = '';
    return $config;
  }


  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    $media_types = MediaType::loadMultiple();
    $options = [];

    foreach ($media_types as $media_type) {
      $options[$media_type->id()] = $media_type->label();
    }
    $form[self::SETTING_MEDIA_TYPE] = [
      '#type' => 'select',
      '#title' => $this->t('Select Media Types'),
      '#default_value' => $this->getSetting(self::SETTING_MEDIA_TYPE),
      '#options' => $options,
    ];

    $form[self::SETTING_MEDIA_FIELD] = [
      '#type' => 'textfield',
      '#title' => $this->t('Whats the media field?'),
      '#default_value' => $this->getSetting(self::SETTING_MEDIA_FIELD),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
  {
    parent::submitConfigurationForm($form, $form_state);
    $this->setConfiguration([
      self::SETTING_MEDIA_TYPE => $form_state->getValue(self::SETTING_MEDIA_TYPE),
      self::SETTING_MEDIA_FIELD => $form_state->getValue(self::SETTING_MEDIA_FIELD),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function tamper($data, TamperableItemInterface $item = NULL)
  {
    if (empty($data)) {
      // No file.
      return $data;
    }
    $fileSystem = \Drupal::service('file_system');
    $file_name = $this->getFileName($fileSystem, $data);
    $media_type = $this->getSetting(self::SETTING_MEDIA_TYPE);
    $media_field = $this->getSetting(self::SETTING_MEDIA_FIELD);
    // Perform a lookup against the value using the configured reference method.

    $file = $this->findFile($file_name);
    $filepath = 'public://' . $file_name;
    if (FALSE == $file) {
      $file = $this->writeData($this->getContent($data), $filepath, 0);
    }

    // Compose file path.
    // $filepath = $this->getDestinationDirectory($fileSystem) . '/' . $file_name;

    if ($file) {
      $media = $this->findMedia($file->id(), $media_field);
      if (!$media) {


        $media = Media::create([
          'name' => $file_name,
          'bundle' => $media_type,
          'uid' => 1,
          'langcode' => 'en',
          'status' => 1,
          $media_field => [
            'target_id' => $file->id(),
            'alt' => $file_name,
            'title' => $file_name,
          ],
        ]);
        $media->save();
      }

      return $media->id();
    }
  }

  protected function getFileName($fileSystem, $url)
  {
    $filename = trim($fileSystem->basename($url), " \t\n\r\0\x0B.");
    // Remove query string from file name, if it has one.
    [$filename] = explode('?', $filename);
    $extension = substr($filename, strrpos($filename, '.') + 1);


    return $filename;
  }


  protected function getContent($url)
  {
    $client = new Client();
    $response = $client->request('GET', $url);

    if ($response->getStatusCode() >= 400) {
      $args = [
        '%url' => $url,
        '@code' => $response->getStatusCode(),
      ];
      return false;
    }

    return (string) $response->getBody();
  }

  protected function writeData($data, $destination = NULL, $replace = FileSystemInterface::EXISTS_RENAME)
  {
    $fileConfig = \Drupal::config('file.settings');
    $fileRepository = \Drupal::service('file.repository');
    if (empty($destination)) {
      $destination = $fileConfig->get('default_scheme') . '://';
    }
    try {
      return $fileRepository->writeData($data, $destination, $replace);
    } catch (EntityStorageException | FileException $e) {
      return FALSE;
    }
  }

  protected function findFile(string $file_name)
  {
    // Check if a file with the same name already exists.
    $existing_files = \Drupal::entityTypeManager()
      ->getStorage('file')
      ->loadByProperties(['filename' => $file_name]);

    if (!empty($existing_files)) {
      // File already exists.
      return reset($existing_files);
    }
    return FALSE;
  }

  protected function findMedia($fid, $media_field)
  {
    // Check if a file with the same name already exists.
    $existing_media = \Drupal::entityTypeManager()
      ->getStorage('media')
      ->loadByProperties([$media_field => $fid]);

    if (!empty($existing_media)) {
      // File already exists.
      return reset($existing_media);
    }
    return FALSE;
  }
}
